<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Infomaker\Everyware\Support\Storage\Cache;
use SplFileInfo;
use Twig_Error;
use Twig_Loader_Chain;
use Twig_Loader_Filesystem;

/**
 * FilesystemLoader
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\FilesystemLoader 0.1
 */
class FilesystemLoader extends Twig_Loader_Filesystem
{

    /**
     * Time in seconds to cache the output if cache is set to true
     *
     * @var int
     */
    private $cacheTime = 86400;

    /**
     * Add suffix so we don't have to delete it in deploy every time
     *
     * @var string
     */
    private $cacheKey = 'everyware_twig_paths_cache';

    private static $instance;

     /**
     * Create new instance if none exist
     *
     * @return ViewSetup
     */
    public static function getInstance(): FilesystemLoader
    {
        if (static::$instance === null) {
            static::$instance = new static;
        }

        return static::$instance;
    }

    /**
     * Adds the folders containing the files to be loaded
     *
     * @param string $folderPath
     * @param string $namespace
     * @param array  $exclude
     *
     * @throws Twig_Error
     * @return void
     */
    public function addPathAndSubfolders(
        string $folderPath,
        string $namespace = Twig_Loader_Filesystem::MAIN_NAMESPACE,
        array $exclude = []
    ): void {

        if ( ! file_exists($folderPath)) {
            throw new Twig_Error("{$folderPath} does not exist");
        }

        $cacheKey = $this->getPathCacheKey($folderPath);
        $paths = $this->getCache($cacheKey);

        if (empty($paths)) {
            $dir = PathFilter::setupFromPath($folderPath, $exclude);
            $paths = [];
            foreach ($dir->filtered() as $d) {
                if ($d instanceof SplFileInfo) {
                    $paths[] = str_replace('//', '/', $d->getPath());
                }
            }

            $this->setCache($cacheKey, $paths);
            $paths = array_unique($paths);
        }

        foreach ($paths as $path) {
            $this->addPath($path, $namespace);
        }
    }
    protected function getCache($cacheKey)
    {
        return Cache::get($cacheKey, []);
    }
    protected function setCache($cacheKey, $paths)
    {
        Cache::set($cacheKey, $paths);
    }
    /**
     *
     *
     * @param $namespace
     *
     * @return string
     */
    protected function getPathCacheKey(string $namespace): string
    {
        if (\defined('GIT_COMMIT')) {
            $namespace .= '_'.GIT_COMMIT;
        }
        return "{$this->cacheKey}_{$namespace}";
    }

    /**
     * Set cache time in seconds
     *
     * @param $time
     *
     * @return void
     */
    public function setCacheTime(int $time): void
    {
        $this->cacheTime = $time;
    }
}
